Attribute VB_Name = "Common"
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Module:      Common
'
'
'-----------------------------------------------------------------------
'
' responsible:  WolfgangA
'
' special area: SQL Studio DB Layer
'
' description:  contains some functions
'
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Option Explicit

'--------------------------------------------------------
'Module Name
'--------------------------------------------------------
Const MODULE_NAME = "Common"


'=public global objects========================================================================================
Public m_oComFunc           As New SQLStudioUtil.Functions
'=========================================================================================

'=========================================================================================
'    function:      internalError
'    description:   shows message box with error
'    arguments:     [in] CurrentError, vb error object
'                   [in] Modulname, name of source modul
'                   [in] FunctionName, name of used function
'    return value:  /
'
'=========================================================================================
Public Function internalError(ByVal CurrentError As ErrObject, ModulName As String, FunctionName As String)
    MsgBox "Module:" + Chr$(9) + ModulName + Chr$(13) + Chr$(10) + "Function:" + Chr$(9) + FunctionName + Chr$(13) + Chr$(10) + "error No:" + Chr$(9) + Trim$(Str$(CurrentError.Number)) + Chr$(13) + Chr$(10) + "error:" + Chr$(9) + CurrentError.Description, vbExclamation, "Unexpected Error:"
End Function

'=========================================================================================
'    function       commonFreeStmt
'    description:   free statement handle
'    arguments:     [in] hStmt, statement handle
'
'    return value:  true if success
'
'=========================================================================================
Public Function commonFreeStmt(hStmt As Long)
    On Error GoTo errCommonFreeStmt

    Const FUNCTION_NAME = "commonFreeStmt"

    Dim lRc  As Integer

    'free statement handle
    If hStmt <> SQL_NULL_HSTMT Then
    
        lRc = SQLFreeStmt(hStmt, SQL_DROP)
        hStmt = SQL_NULL_HSTMT
    End If

    Exit Function

errCommonFreeStmt:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      getUID
'    description:   gets the user ID out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sUID
'
'    return value:  /
'
'=========================================================================================
Public Function getUID(ByVal ConnectString As String, ByRef sUID As String)
    On Error GoTo errGetUID

    Const FUNCTION_NAME = "getUID"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '=====================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";UID=")
    If BeginPos > 0 Then
    
        If Len(ConnectString) > BeginPos + 5 Then
        
            EndPos = InStr(BeginPos, ConnectString, ";PWD=")
            If EndPos > 0 Then
            
                sUID = Mid$(ConnectString, BeginPos + 5, EndPos - BeginPos - 5)
                
                If left$(sUID, 1) = Chr$(34) And right$(sUID, 1) = Chr$(34) And Len(sUID) > 2 Then
                    sUID = Mid$(sUID, 2, Len(sUID) - 2)
                End If
            End If
        End If
    Else
        'in a new version  odbc changed output of connect string
        'see PTS error 1108230
        BeginPos = InStr(ConnectString, "UID=")
        If Len(ConnectString) > BeginPos + 4 Then
        
            EndPos = InStr(BeginPos, ConnectString, ";PWD=")
            If EndPos > 0 Then
            
                sUID = Mid$(ConnectString, BeginPos + 4, EndPos - BeginPos - 4)
                
                If left$(sUID, 1) = Chr$(34) And right$(sUID, 1) = Chr$(34) And Len(sUID) > 2 Then
                    sUID = Mid$(sUID, 2, Len(sUID) - 2)
                End If
            End If
        End If
    End If
    
    Exit Function

errGetUID:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getPWD
'    description:   gets the password out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sPWD
'
'    return value:  /
'
'=========================================================================================
Public Function getPWD(ByVal ConnectString As String, ByRef sPWD As String)
    On Error GoTo errGetPWD

    Const FUNCTION_NAME = "getPWD"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '======================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";PWD=")
    If BeginPos > 0 Then
        If Len(ConnectString) > BeginPos + 5 Then
            EndPos = InStr(BeginPos, ConnectString, ";SERVERDB=")
            If EndPos > 0 Then
                sPWD = Mid$(ConnectString, BeginPos + 5, EndPos - BeginPos - 5)
                
                If left$(sPWD, 1) = Chr$(34) And right$(sPWD, 1) = Chr$(34) And Len(sPWD) > 2 Then
                    sPWD = Mid$(sPWD, 2, Len(sPWD) - 2)
                End If
            End If
        End If
    End If
    
    Exit Function

errGetPWD:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getServerDB
'    description:   gets the server DB out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sServerDB
'
'    return value:  /
'
'=========================================================================================
Public Function getServerDB(ByVal ConnectString As String, ByRef sServerDB As String)
    On Error GoTo errGetServerDB

    Const FUNCTION_NAME = "getServerDB"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '======================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";SERVERDB=")
    If BeginPos > 0 Then
        If Len(ConnectString) > BeginPos + 10 Then
            EndPos = InStr(BeginPos, ConnectString, ";SERVERNODE=")
            If EndPos > 0 Then
            
                sServerDB = Mid$(ConnectString, BeginPos + 10, EndPos - BeginPos - 10)
            End If
        End If
    End If
    
    Exit Function

errGetServerDB:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      getServerNode
'    description:   gets the server Node out of a odbc connection string
'    arguments:     [in] ConnectString
'                   [out] sServerNode
'
'    return value:  /
'
'=========================================================================================
Public Function getServerNode(ByVal ConnectString As String, ByRef sServerNode As String)
    On Error GoTo errGetServerNode

    Const FUNCTION_NAME = "getServerNode"
    
    '=varibles============================================================================
    Dim BeginPos    As Integer
    Dim EndPos      As Integer
    '======================================================================================

    'get first position
    BeginPos = InStr(ConnectString, ";SERVERNODE=")
    If BeginPos > 0 Then
    
        If Len(ConnectString) > BeginPos + 12 Then
        
            EndPos = InStr(BeginPos + 1, ConnectString, ";")
            If EndPos > 0 Then
            
                sServerNode = Mid$(ConnectString, BeginPos + 12, EndPos - BeginPos - 12)
            End If
        End If
    End If
    
    Exit Function

errGetServerNode:
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
'=========================================================================================
'    function:      displaySQLOrError
'    description:   Sql commands or erros are displayed in a status window.
'                   The status window has to be from the type SQLSTUDIODSQLLibCtl.SQLStudioDSQLInput.
'                   If only errors or all sql commands are displayed is driven
'                   by the verbose flag of the connection.
'    arguments:     /
'
'    return value:  /
'
'=========================================================================================
Public Function displaySQLOrError(oConnection As connection, _
                                  ByVal lRc As Long, _
                                  ByVal lCallingWindow, _
                                  ByVal sTimeDiff As String)
    On Error GoTo errDisplaySQLOrError

    Const FUNCTION_NAME = "displaySQLOrError"
    
    '=variables=============================================================================
    Dim Pos1           As Long
    Dim Pos2           As Long
    Dim lCut           As Long
    Dim lErrorHeader   As Long
    Dim sLastText      As String
    Dim lSelStart      As Long
    Dim lSelLength     As Long
    Dim lLastLength    As Long
    Dim lErrorStart    As Long
    Dim sStatement     As String
    Dim sProtText      As String
    Dim sErrorHeader   As String
    Dim textLimit      As Long
    Dim buffer         As String
    '=======================================================================================
        
    'check if verbosre mode should display non error statements
    If oConnection.statusVerbose = False And lRc = 0 Then Exit Function
        
    'check for valid sql statement
    sStatement = oConnection.error.statement
    If (Len(sStatement) < 1) Then Exit Function
    
    sLastText = oConnection.error.lastText
    lCut = InStr(sLastText, "[SQLOD32 DLL]")
    
    If lCut > 0 Then
        sLastText = right(sLastText, Len(sLastText) - lCut - 12)
        lCut = InStr(sLastText, "]")
        If lCut > 0 Then sLastText = right(sLastText, Len(sLastText) - lCut)
    End If
    
    'make window writeable
    oConnection.protWindow().ReadOnly = False
    
    lLastLength = oConnection.protWindow().textLength
    textLimit = oConnection.protWindow().textLimit
    'if text is to big cut of first third
    If lLastLength >= textLimit - 500 Then
        sProtText = oConnection.protWindow().text
        sProtText = right(sProtText, textLimit \ 3)
        oConnection.protWindow().text = sProtText
    End If
    
    If lRc = SQL_SUCCESS Then
    
        'display last statement
        oConnection.protWindow().appendText ("++++ " + LoadResString(119) + " +++++++++++++++++++++++++++++++" + vbCrLf)
    Else
        sProtText = oConnection.protWindow().text
        'get 'real' text len see comments WM_GETTEXTLENGTH in MSDN
        'When the WM_GETTEXTLENGTH message is sent, the DefWindowProc function
        'returns the length, in characters, of the text.
        'Under certain conditions, the DefWindowProc function returns a
        'value that is larger than the actual length of the text.
        'This occurs with certain mixtures of ANSI and Unicode ...
        'With CrLf in the text the function len() returns the right length
        'to work for coloring the text
        lErrorStart = Len(sProtText)
        
        'set text color first
        oConnection.protWindow().setRangeTextColor lErrorStart - 1, lErrorStart - 1, RGB(0, 0, 0)
        
        'add header line
        sErrorHeader = "---- " + LoadResString(120) + " -------------------------------"
        oConnection.protWindow().appendText (sErrorHeader + vbCrLf)
        
        lErrorHeader = Len(sErrorHeader)
        'color sql error begining
        oConnection.protWindow().setRangeTextColor lErrorStart - 1, lErrorStart + lErrorHeader, RGB(255, 0, 0)
        'set text color back
        oConnection.protWindow().setRangeTextColor lErrorStart + lErrorHeader, lErrorStart + lErrorHeader + 1, RGB(0, 0, 0)
        'set cursor to begin of error
        oConnection.protWindow().SetSelectionIndexes lErrorStart - 1, lErrorStart - 1
    End If
    
    
'    If (lCallingWindow <> 0) Then
'        buffer = Space$(cnstWindowTitleLength)
'        GetWindowText lCallingWindow, buffer, Len(buffer)
'        oConnection.protWindow().appendText (buffer)
'
'    Else
'        oConnection.protWindow().appendText (LoadResString(128))
'    End If
    
    'oConnection.protWindow().appendText (LoadResString(129))
    
    'oConnection.protWindow().appendText (": ")
    oConnection.protWindow().appendText (oConnection.autoCommitAsString)
    oConnection.protWindow().appendText (", ")
    oConnection.protWindow().appendText (LoadResString(326) + oConnection.sqlModeAsString)
    oConnection.protWindow().appendText (", ")
    oConnection.protWindow().appendText (LoadResString(306) + oConnection.isoLevelAsString + vbCrLf)
    
            
    If lRc = SQL_SUCCESS Then
    
        oConnection.protWindow().appendText (sStatement)
        If right$(sStatement, 2) <> vbCrLf And right$(sStatement, 1) <> vbCr Then
            oConnection.protWindow().appendText (vbCrLf)
        End If
        If Len(sTimeDiff) > 0 Then
            oConnection.protWindow().appendText (sTimeDiff + vbCrLf)
        End If
        'oConnection.protWindow().appendText (LoadResString(109) + vbCrLf)
    Else
        'error text
        oConnection.protWindow().appendText (sLastText + vbCrLf)
        
        'statement
        'oConnection.protWindow().appendText (LoadResString(130) + ":" + vbCrLf)
        
        sStatement = oConnection.error.lastStatement
        'get last text len
        sProtText = oConnection.protWindow().text
        'get 'real' text len see comments WM_GETTEXTLENGTH in MSDN
        'When the WM_GETTEXTLENGTH message is sent, the DefWindowProc function
        'returns the length, in characters, of the text.
        'Under certain conditions, the DefWindowProc function returns a
        'value that is larger than the actual length of the text.
        'This occurs with certain mixtures of ANSI and Unicode ...
        'With CrLf in the text the function len() returns the right length
        'to work for coloring the text
        lLastLength = Len(sProtText)
        
        'add error sql statement
        oConnection.protWindow().appendText (sStatement)
        If right$(sStatement, 2) <> vbCrLf And right$(sStatement, 1) <> vbCr Then
            oConnection.protWindow().appendText (vbCrLf)
        End If
        'make error position
        If oConnection.error.lastIsError Then
            If Len(sStatement) > 0 And Len(oConnection.error.lastPos) > 0 Then

                If oConnection.error.lastPos = "0" Then
                    Pos1 = InStr(1, sStatement, Chr$(32))
                    Pos2 = InStr(1, sStatement, Chr$(13))
                Else
                    Pos1 = InStr(oConnection.error.lastPos, sStatement, Chr$(32))
                    Pos2 = InStr(oConnection.error.lastPos, sStatement, Chr$(13))
                End If

                If oConnection.error.lastPos > 0 Then
                    lSelStart = oConnection.error.lastPos - 2
                Else
                    lSelStart = 0
                End If

                If Pos1 <> 0 Then
                    If Pos1 < Pos2 Or Pos2 = 0 Then
                        If Pos1 > lSelStart Then
                            lSelLength = Pos1 - lSelStart - 1
                        End If
                    Else
                        If Pos2 > lSelStart Then
                            lSelLength = Pos2 - lSelStart - 1
                        End If
                    End If
                ElseIf Pos2 <> 0 Then
                    If Pos2 > lSelStart Then
                        lSelLength = Pos2 - lSelStart - 1
                    End If
                Else
                    If Len(sStatement) > lSelStart Then
                        lSelLength = Len(sStatement) - lSelStart
                    End If
                End If
            End If
        End If
        'native error
        'oConnection.protWindow().appendText (LoadResString(126) + ": " + oConnection.error.lastNativeError + vbCrLf)
        'error pos
        'oConnection.protWindow().appendText (LoadResString(123) + ": " + oConnection.error.lastPos) + vbCrLf
        'error time
        'oConnection.protWindow().appendText (LoadResString(124) + ": " + oConnection.error.lastTime) + vbCrLf
        'error rc
        'oConnection.protWindow().appendText (LoadResString(122) + ": " + oConnection.error.lastRC) + vbCrLf
        'sql state
        'oConnection.protWindow().appendText (LoadResString(121) + ": " + oConnection.error.lastSQLState) + vbCrLf
        
        'color sql error
        oConnection.protWindow().setRangeTextColor lLastLength + lSelStart, lLastLength + lSelStart + lSelLength - 1, RGB(255, 0, 0)
        'set cursor to begin of error
        oConnection.protWindow().SetSelectionIndexes lErrorStart - 1, lErrorStart - 1
    End If
    
    oConnection.protWindow().ReadOnly = True
    
    Exit Function
    
errDisplaySQLOrError:
    oConnection.protWindow().ReadOnly = True
    internalError err, MODULE_NAME, FUNCTION_NAME
End Function
